import { Server } from 'socket.io'
import { TSocket } from '../../../utilities/types'
import ZoneRepository from '../../../repositories/zoneRepository'
import prisma from '../../../utilities/prisma'

type Payload = {
  zoneId: number
}

/**
 * Handle game master zone delete event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('gm:zone_editor:zone:delete', async (data: Payload, callback: (response: boolean) => void) => {
    if (socket.character?.role !== 'gm') {
      console.log(`---Character #${socket.character?.id} is not a game master.`)
      return
    }

    console.log(`---GM ${socket.character?.id} has deleted a zone via zone editor.`)

    try {
      const zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        console.log(`---Zone not found.`)
        return
      }

      await prisma.zone.delete({
        where: {
          id: data.zoneId
        }
      })

      callback(true)
    } catch (e) {
      console.error(e)
      callback(false)
    }
  })
}
