import { Server } from 'socket.io'
import { TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import ZoneManager from '../../managers/zoneManager'
import { Character, Zone } from '@prisma/client'
import CharacterManager from '../../managers/characterManager'
import logger from '../../utilities/logger'

export default class ZoneLeaveEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('zone:character:leave', this.handleZoneLeave.bind(this))
  }

  private async handleZoneLeave(): Promise<void> {
    try {
      if (!this.socket.character) {
        logger.error('zone:character:leave error', 'Character not found')
        return
      }

      if (!this.socket.character.zoneId) {
        logger.error('zone:character:leave error', 'Character not in a zone')
        return
      }

      const zone = await ZoneRepository.getById(this.socket.character.zoneId)

      if (!zone) {
        logger.error('zone:character:leave error', 'Zone not found')
        return
      }

      this.socket.leave(zone.id.toString())

      // let other clients know of character leaving
      this.io.to(zone.id.toString()).emit('zone:character:leave', this.socket.character.id)

      // remove character from zone manager
      await CharacterManager.removeCharacter(this.socket.character)

      logger.info('zone:character:leave', `Character ${this.socket.character.id} left zone ${zone.id}`)
    } catch (error: any) {
      logger.error('zone:character:leave error', error.message)
    }
  }
}
